#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/8/20

from mtdata.index import Entry, Index

def load(index: Index):
    # === Para crawl corpus
    PARACRAWL_v3 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release3/%s-%s.bicleaner07.tmx.gz'
    cite = r"""@inproceedings{espla-etal-2019-paracrawl,
        title = "{P}ara{C}rawl: Web-scale parallel corpora for the languages of the {EU}",
        author = "Espl{\`a}, Miquel  and
          Forcada, Mikel  and
          Ram{\'\i}rez-S{\'a}nchez, Gema  and
          Hoang, Hieu",
        booktitle = "Proceedings of Machine Translation Summit XVII Volume 2: Translator, Project and User Tracks",
        month = aug,
        year = "2019",
        address = "Dublin, Ireland",
        publisher = "European Association for Machine Translation",
        url = "https://www.aclweb.org/anthology/W19-6721",
        pages = "118--119",
    }
@inproceedings{banon-etal-2020-paracrawl,
    title = "{P}ara{C}rawl: Web-Scale Acquisition of Parallel Corpora",
    author = "Ba{\~n}{\'o}n, Marta  and
      Chen, Pinzhen  and
      Haddow, Barry  and
      Heafield, Kenneth  and
      Hoang, Hieu  and
      Espl{\`a}-Gomis, Miquel  and
      Forcada, Mikel L.  and
      Kamran, Amir  and
      Kirefu, Faheem  and
      Koehn, Philipp  and
      Ortiz Rojas, Sergio  and
      Pla Sempere, Leopoldo  and
      Ram{\'\i}rez-S{\'a}nchez, Gema  and
      Sarr{\'\i}as, Elsa  and
      Strelec, Marek  and
      Thompson, Brian  and
      Waites, William  and
      Wiggins, Dion  and
      Zaragoza, Jaume",
    booktitle = "Proceedings of the 58th Annual Meeting of the Association for Computational Linguistics",
    month = jul,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/2020.acl-main.417",
    doi = "10.18653/v1/2020.acl-main.417",
    pages = "4555--4567",
}    
    """
    for pair in ['en cs', 'en de', 'en fi', 'en lt']:
        l1, l2 = pair.split()
        index.add_entry(
            Entry(langs=(l1, l2), name='paracrawl_v3', url=PARACRAWL_v3 % (l1, l2), cite=cite))

    # === Paracrawl V6
    PARACRAWL_v6 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release6/%s-%s.txt.gz'
    for l2 in ['is', 'bg', 'hr', 'cs', 'da', 'nl', 'et', 'fi', 'fr', 'de', 'el', 'hu', 'ga', 'it', 'lv',
               'lt', 'mt', 'pl', 'pt', 'ro', 'sk', 'sl', 'es', 'sv']:
        l1 = 'en'
        index.add_entry(Entry(langs=(l1, l2), name='paracrawl_v6', url=PARACRAWL_v6 % (l1, l2),
                             cite=cite, ext='tsv.gz'))
    # these are bonus
    PARACRAWL_v6_B = 'https://s3.amazonaws.com/web-language-models/paracrawl/release6/%s-%s.bicleaner07.txt.gz'
    for l1, l2 in [('nl', 'fr'), ('pl', 'de')]:
        index.add_entry(Entry(langs=(l1, l2), name='paracrawl_v6', url=PARACRAWL_v6_B % (l1, l2),
                             cite=cite, ext='tsv.gz'))

    l1 = 'en'
    PARACRAWL_v7_1 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release7.1/%s-%s.txt.gz'
    for l2 in 'bg cs da de el es et fi fr ga hr hu is it lt lv mt nl pl pt ro sk sl sv'.split():
        index.add_entry(Entry(langs=(l1, l2), name='paracrawl_v7_1', url=PARACRAWL_v7_1 % (l1, l2),
                              cite=cite, ext='tsv.gz'))
    PARACRAWL_v7_1 = 'https://s3.amazonaws.com/web-language-models/paracrawl/release7/%s-%s.txt.gz'
    for pair in 'en-nb en-nn es-ca es-eu es-gl'.split():
        l1, l2 = pair.split('-')
        l2 = dict(nb='nob').get(l2, l2)
        index.add_entry(Entry(langs=(l1, l2), name='paracrawl_v7', url=PARACRAWL_v7_1 % (l1, l2),
                              cite=cite, ext='tsv.gz'))



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/13/20
import gzip
from mtdata import log
import shutil
from datetime import datetime
from pathlib import Path


class IO:
    """File opener and automatic closer
    Copied from my other project https://github.com/isi-nlp/rtg/blob/master/rtg/utils.py
    """
    def __init__(self, path, mode='r', encoding=None, errors=None):
        self.path = path if type(path) is Path else Path(path)
        self.mode = mode
        self.fd = None
        self.encoding = encoding or 'utf-8' if 't' in mode else None
        self.errors = errors or 'replace'

    def __enter__(self):
        if self.path.name.endswith(".gz"):   # gzip mode
            self.fd = gzip.open(self.path, self.mode, encoding=self.encoding, errors=self.errors)
        elif self.path.name.endswith(".xz"):
            import lzma
            self.fd = lzma.open(self.path, self.mode, encoding=self.encoding, errors=self.errors)
        else:
            if 'b' in self.mode:  # binary mode doesnt take encoding or errors
                self.fd = self.path.open(self.mode)
            else:
                self.fd = self.path.open(self.mode, encoding=self.encoding, errors=self.errors,
                                         newline='\n')
        return self.fd

    def __exit__(self, _type, value, traceback):
        self.fd.close()
        self.fd = None

    @classmethod
    def reader(cls, path, text=True):
        return cls(path, 'rt' if text else 'rb')

    @classmethod
    def writer(cls, path, text=True, append=False):
        return cls(path, ('a' if append else 'w') + ('t' if text else 'b'))

    @classmethod
    def get_lines(cls, path, col=0, delim='\t', line_mapper=None, newline_fix=True):
        with cls.reader(path) as inp:
            if newline_fix and delim != '\r':
                inp = (line.replace('\r', '') for line in inp)
            if col >= 0:
                inp = (line.split(delim)[col].strip() for line in inp)
            if line_mapper:
                inp = (line_mapper(line) for line in inp)
            yield from inp

    @classmethod
    def get_liness(cls, *paths, **kwargs):
        for path in paths:
            yield from cls.get_lines(path, **kwargs)

    @classmethod
    def write_lines(cls, path: Path, text):
        if isinstance(text, str):
            text = [text]
        with cls.writer(path) as out:
            for line in text:
                out.write(line)
                out.write('\n')

    @classmethod
    def copy_file(cls, src: Path, dest: Path, text=False):
        assert src.resolve() != dest.resolve()
        log.info(f"Copy {src} → {dest}")
        with IO.reader(src, text=text) as inp, IO.writer(dest, text=text) as out:
            shutil.copyfileobj(inp, out)

    @classmethod
    def maybe_backup(cls, file: Path):
        if file.exists():
            time = datetime.now().strftime('%Y%m%d_%H%M%S_%f')
            dest = file.with_suffix(f'.{time}')
            log.info(f"Backup {file} → {dest}")
            file.rename(dest)


#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 1/26/21

import mtdata
from mtdata.main import get_data
from argparse import Namespace
from pathlib import Path
from tempfile import TemporaryDirectory


def test_rapid1019():
    #out_dir = Path('tmp-test-tmx1')
    with TemporaryDirectory() as out_dir:
        out_dir = Path(out_dir)
        args = Namespace(cache=mtdata.cache_dir, langs=('eng', 'ces'), merge=False,
                         out=out_dir, task='get', test_names=None, train_names=['rapid2019'])
        get_data(args)
        parts_dir = out_dir / 'train-parts'
        eng = parts_dir / 'rapid2019-eng_ces.eng'
        ces = parts_dir / 'rapid2019-eng_ces.ces'
        eng = eng.read_text().splitlines()
        ces = ces.read_text().splitlines()
        assert len(eng) == len(ces) == 263_287
        assert eng[0].strip() == '(^) Products that are put to sale before 20 May 2016 can still be sold in the EU until 2017.'
        assert ces[0].strip() =='(^) Výrobky, které byly určeny k prodeji před 20. květnem 2016, mohou být v EU stále prodávány do roku 2017.'


test_rapid1019()


from mtdata import log
from mtdata.entry import Entry

from dataclasses import dataclass
from typing import TextIO, List
import abc
from collections import defaultdict

class Reporter(abc.ABC):

    @abc.abstractmethod
    def report(self, entries: List[Entry], out: TextIO, **kwargs):
        raise NotImplementedError()

@dataclass
class PlainReporter(Reporter):
    """Writes plain text output"""
    field_sep: str = '\t'
    rec_sep = '\n'
    full: bool = False


    def report(self, entries, out: TextIO):
        mem = defaultdict()

        for i, ent in enumerate(entries):
            out.write(ent.format(delim=self.field_sep))
            out.write(self.rec_sep)
            if self.full:
                out.write(ent.cite or "CITATION_NOT_LISTED")
                out.write(self.rec_sep)
                out.write(self.rec_sep)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/13/20

from mtdata.index import Index, Entry
import itertools

def load_all(index: Index):
    cite="""@inproceedings{ziemski-etal-2016-united,
    title = "The United Nations Parallel Corpus v1.0",
    author = "Ziemski, Micha{\l}  and
      Junczys-Dowmunt, Marcin  and
      Pouliquen, Bruno",
    booktitle = "Proceedings of the Tenth International Conference on Language Resources and Evaluation ({LREC}'16)",
    month = may,
    year = "2016",
    address = "Portoro{\v{z}}, Slovenia",
    publisher = "European Language Resources Association (ELRA)",
    url = "https://www.aclweb.org/anthology/L16-1561",
    pages = "3530--3534",
}"""
    url = "https://stuncorpusprod.blob.core.windows.net/corpusfiles/UNv1.0.testsets.tar.gz"
    langs = ['en', 'ar', 'fr', 'es', 'ru', 'zh']
    for split in ['dev', 'test']:
        for l1, l2 in itertools.combinations(langs, 2):
            f1 = f'testsets/{split}set/UNv1.0.{split}set.{l1}'
            f2 = f'testsets/{split}set/UNv1.0.{split}set.{l2}'
            ent = Entry(langs=(l1, l2), url=url, filename='UNv1.0.testsets.tar.gz', name=f'UNv1_{split}', in_ext='txt',
                        in_paths=[f1, f2], cite=cite)
            index.add_entry(ent)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 8/27/20


from mtdata.index import Index, Entry


class NoisyEntry(Entry):

    def is_NULL(self, seg):
        return  '__NULL__' in seg or '_ _ NULL _ _' in seg

    def is_noisy(self, seg1, seg2) -> bool:
        noise = super().is_noisy(seg1, seg2) or self.is_NULL(seg1) or self.is_NULL(seg2)
        return noise


def load_all(index: Index):
    url = "http://phontron.com/data/ted_talks.tar.gz"
    cite = """@inproceedings{Ye2018WordEmbeddings,
    author  = {Ye, Qi and Devendra, Sachan and Matthieu, Felix and Sarguna, Padmanabhan and Graham, Neubig},
    title   = {When and Why are pre-trained word embeddings useful for Neural Machine Translation},
    booktitle = {HLT-NAACL},
    year    = {2018},
    }"""
    header = (
        "-,en,es,pt-br,fr,ru,he,ar,ko,zh-cn,it,ja,zh-tw,nl,ro,tr,de,vi,pl,pt,bg,el,fa,sr,hu,hr,"
        "uk,cs,id,th,sv,sk,sq,lt,da,calv-,my,sl,mk,fr-ca,fi,hy,hi,nor,ka,mn,et,ku,gl,mr,zh,ur,"
        "eo,ms,az,ta,bn,kk,be,eu,bs").split(',')
    col_idx = {lang: idx for idx, lang in enumerate(header)}

    # langs that I care; exclude <lang>-<country> bcoz the iso3 code doesnt have a way to map it
    langs = [x for x in header if '-' not in x]
    for split in ['train', 'test', 'dev']:
        for idx1, lang1 in enumerate(langs):
            col1 = col_idx[lang1]
            for lang2 in langs[idx1 + 1:]:
                col2 = col_idx[lang2]
                ent = NoisyEntry(langs=(lang1, lang2),
                                 name=f"neulab_tedtalksv1_{split}",
                                 filename='neulab_ted_talksv1.tar.gz',
                                 url=url, in_paths=[f"all_talks_{split}.tsv"],
                                 cols=(col1, col2), cite=cite)
                index.add_entry(ent)



# -*- coding: utf-8 -*-
#
# Author: Thamme Gowda [tg (at) isi (dot) edu]
# Created: 4/7/20

# Define here the models for your scraped items
#
# See documentation in:
# http://doc.scrapy.org/en/latest/topics/items.html

import scrapy
from scrapy import Field


class MTDataItem(scrapy.Item):

    # define the fields for your item here like:
    # name = scrapy.Field()
    id = Field()
    name = Field()
    url = Field()
    langs = Field()
    title = Field()
    info = Field()

class MTCorpusItem(scrapy.Item):
    url = Field()
    name = Field()
    langs = Field()




#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/4/20

from typing import Optional, Union, Tuple, List
from dataclasses import dataclass
from pathlib import Path
from mtdata import log
from itertools import zip_longest

from mtdata.utils import IO

COMPRESS_EXT = ['gz', 'bz2', 'xz']


def detect_extension(name: Union[str, Path]):
    """
    detects file extension from file name
    :param filename: name or full path of file
    :return: extension of file
    """

    filename = name if isinstance(name, str) else name.name
    parts = filename.split('.')
    ext = parts[-1]
    if ext in COMPRESS_EXT:
        ext = '.'.join(parts[-2:])
    return ext


@dataclass
class Parser:
    paths: Union[Path, List[Path]]
    langs: Tuple[str, str]
    ext: Optional[str] = None
    ent: Optional['Entry'] = None

    def __post_init__(self):
        if not isinstance(self.paths, list):
            self.paths = [self.paths]
        assert 1 <= len(self.paths) <= 3
        for p in self.paths:
            assert p.exists(), f'{p} not exists'

        if not self.ext:
            exts = [detect_extension(p.name) for p in self.paths]
            if len(exts) == 2 and set(exts) == set(self.langs):
                log.warning(f"Treating {' .'.join(exts)} as plain text. To override: in_ext=<ext>")
                exts = ['txt']  # treat that as plain text
            assert len(set(exts)) == 1, f'Expected a type of exts, but found: {exts}'
            self.ext = exts[0]

    def read_segs(self):
        readers = []
        if self.ext == 'opus_xces':
            align, lang1_dir, lang2_dir = self.paths
            from mtdata.opus_xces import OpusXcesParser
            reader = OpusXcesParser.read(align, lang1_dir, lang2_dir)
            readers.append(reader)
        else:
            for p in self.paths:
                if 'tsv' in self.ext:
                    cols = (0, 1) #extract first two columns
                    if self.ent and self.ent.cols:
                        cols = self.ent.cols
                    readers.append(self.read_tsv(p, cols=cols))
                elif 'raw' in self.ext or 'txt' in self.ext:
                    readers.append(self.read_plain(p))
                elif 'tmx' in self.ext:
                    from mtdata.tmx import read_tmx
                    readers.append(read_tmx(path=p, langs=self.langs))
                elif 'sgm' in self.ext:
                    from mtdata.sgm import read_sgm
                    readers.append(read_sgm(p))
                else:
                    raise Exception(f'Not supported {self.ext} : {p}')

        if len(readers) == 1:
            yield from readers[0]
        elif len(readers) == 2:
            for seg1, seg2 in zip_longest(*readers):
                if seg1 is None or seg2 is None:
                    raise Exception(f'{self.paths} have unequal number of segments')
                yield seg1, seg2
        else:
            raise Exception("This is an error")

    def read_plain(self, path):
        with IO.reader(path) as stream:
            for line in stream:
                yield line.strip()

    def read_tsv(self, path, delim='\t', cols=None):
        """
        Read data from TSV file
        :param path: path to TSV file
        :param delim: delimiter default is \\t
        :param cols: if certain columns are to be extracted;
            default is None, which returns all columns
        :return:
        """
        with IO.reader(path) as stream:
            for line in stream:
                row = [x.strip() for x in line.rstrip('\n').split(delim)]
                if cols:
                    row = [row[idx] for idx in cols]
                yield row



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 4/5/20

from typing import Union
from pathlib import Path
from xml.etree import ElementTree as ET
import argparse
from mtdata import log
from mtdata.utils import IO
import time
from mtdata.iso import iso3_code
from html import unescape
import datetime


DEF_PROGRESS = 10  # seconds

def parse_tmx(data, n_langs=2, log_every=DEF_PROGRESS):
    context = ET.iterparse(data, events=['end'])
    tus = (el for event, el in context if el.tag == 'tu')
    count, skips = 0, 0
    st = t = time.time()
    for tu in tus:
        langs, segs = [], []
        for tuv in tu.findall('tuv'):
            lang = [v for k, v in tuv.attrib.items() if k.endswith('lang')]
            if lang:
                langs.append(lang[0])
            seg = tuv.findtext('seg')
            if seg:
                segs.append(unescape(seg.strip()))
        if n_langs and len(segs) == len(langs) == n_langs:
            count += 1
            yield list(zip(langs, segs))
        else:
            skips += 1
            log.warning(f"Skipped: langs {langs} segs {len(segs)} ; Parsed count {count}")
        if log_every and (time.time() - t) > log_every:
            elapsed = datetime.timedelta(seconds=round(time.time() - st))
            log.info(f"{elapsed} :: Parsed: {count:,} Skipped:{skips:,}")
            t = time.time()
        tu.clear()
    log.info(f"Skipped ={skips}; parsed: {count}")

def read_tmx(path: Union[Path], langs=None):
    """
    reads a TMX file as records
    :param path: path to .tmx file
    :param langs: (lang1, lang2) codes eg (de, en); when it is None the code tries to auto detect
    :return: stream of (text1, text2)
    """
    with IO.reader(path) as data:
        recs = parse_tmx(data)
        for rec in recs:
            (l1, t1), (l2, t2) = rec
            l1 = iso3_code(l1, fail_error=True)
            l2 = iso3_code(l2, fail_error=True)
            if langs is None:
                log.warning("langs not set; this could result in language mismatch")
                langs = (l1, l2)
            if l1 == langs[0] and l2 == langs[1]:
                yield t1, t2
            elif l2 == langs[0] and l1 == langs[1]:
                yield t2, t1
            else:
                raise Exception(f"Language code mismatch;; ({l1, l2}) != {langs}")

def main(inp, out):
    recs = read_tmx(inp)
    with IO.writer(out) as out:
        count = 0
        for rec in recs:
            rec = [l.replace('\t', ' ') for l in rec]
            out.write('\t'.join(rec) + '\n')
            count += 1
        log.warning(f"Wrote {count} lines to {out}")


if __name__ == '__main__':
    p = argparse.ArgumentParser(description='A tool to convert TMX to TSV',
                                formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    p.add_argument('-i', '--inp', type=Path, required=True, help='Input file path')
    p.add_argument('-o', '--out', type=Path, default=Path('/dev/stdout'),
                   help='Output file path')
    args = vars(p.parse_args())
    main(**args)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 9/12/20
from mtdata.index import Index, Entry
supervised_v1 = (
    "af-en,am-en,an-en,ar-en,as-en,az-en,be-en,bg-en,bn-en,br-en,bs-en,ca-en,cs-en,cy-en,da-en,"
    "de-en,dz-en,el-en,en-eo,en-es,en-et,en-eu,en-fa,en-fi,en-fr,en-fy,en-ga,en-gd,en-gl,en-gu,"
    "en-ha,en-he,en-hi,en-hr,en-hu,en-hy,en-id,en-ig,en-is,en-it,en-ja,en-ka,en-kk,en-km,en-kn,"
    "en-ko,en-ku,en-ky,en-li,en-lt,en-lv,en-mg,en-mk,en-ml,en-mn,en-mr,en-ms,en-mt,en-my,en-nb,"
    "en-ne,en-nl,en-nn,en-no,en-oc,en-or,en-pa,en-pl,en-ps,en-pt,en-ro,en-ru,en-rw,en-se,en-sh,"
    "en-si,en-sk,en-sl,en-sq,en-sr,en-sv,en-ta,en-te,en-tg,en-th,en-tk,en-tr,en-tt,en-ug,en-uk,"
    "en-ur,en-uz,en-vi,en-wa,en-xh,en-yi,en-yo,en-zh,en-zu").split(",")
zeroshot_v1 = (
    "ar-de,ar-fr,ar-nl,ar-ru,ar-zh,de-fr,de-nl,de-ru,de-zh,fr-nl,fr-ru,fr-zh,nl-ru,nl-zh,ru-zh"
).split(",")


def load_all(index: Index):
    URL = "https://object.pouta.csc.fi/OPUS-100/v1.0/opus-100-corpus-v1.0.tar.gz"
    cite="""
@inproceedings{zhang-etal-2020-improving,
    title = "Improving Massively Multilingual Neural Machine Translation and Zero-Shot Translation",
    author = "Zhang, Biao  and
      Williams, Philip  and
      Titov, Ivan  and
      Sennrich, Rico",
    booktitle = "Proceedings of the 58th Annual Meeting of the Association for Computational Linguistics",
    month = jul,
    year = "2020",
    address = "Online",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/2020.acl-main.148",
    doi = "10.18653/v1/2020.acl-main.148",
    pages = "1628--1639",
}
@inproceedings{tiedemann2012parallel,
  title={Parallel Data, Tools and Interfaces in OPUS.},
  author={Tiedemann, J{\"o}rg},
  booktitle={Lrec},
  volume={2012},
  pages={2214--2218},
  year={2012}
}"""
    filename = 'opus-100-corpus-v1.0.tar.gz'
    code_map = dict(nb='nob', sh='hbs')  # these arent obvious to iso lookup function, so helping
    for pair in supervised_v1:
        l1, l2 = pair.split("-")
        l1 = code_map.get(l1, l1)
        l2 = code_map.get(l2, l2)
        splits = ['train', 'dev', 'test']
        if pair in {'an-en', 'en-yo', 'dz-en', 'en-hy', 'en-mn'}:
            splits = ['train']  # somehow they forgot to include test sets for these
        for split in splits:
            f1 = f'opus-100-corpus/v1.0/supervised/{l1}-{l2}/opus.{l1}-{l2}-{split}.{l1}'
            f2 = f'opus-100-corpus/v1.0/supervised/{l1}-{l2}/opus.{l1}-{l2}-{split}.{l2}'
            ent = Entry(langs=(l1, l2), url=URL, name=f'OPUS100v1_{split}',
                        filename=filename, in_paths=[f1, f2], in_ext='txt', cite=cite)
            index.add_entry(ent)
    for pair in zeroshot_v1:
        l1, l2 = pair.split("-")
        f1 = f'opus-100-corpus/v1.0/zero-shot/{l1}-{l2}/opus.{l1}-{l2}-test.{l1}'
        f2 = f'opus-100-corpus/v1.0/zero-shot/{l1}-{l2}/opus.{l1}-{l2}-test.{l2}'
        ent = Entry(langs=(l1, l2), url=URL, name=f'OPUS100v1_test', filename=filename,
                    in_paths=[f1, f2], in_ext='txt', cite=cite)
        index.add_entry(ent)


#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/13/20
from mtdata.index import Index, Entry

def load_all(index: Index):

    cite="""@inproceedings{post-etal-2012-constructing,
    title = "Constructing Parallel Corpora for Six {I}ndian Languages via Crowdsourcing",
    author = "Post, Matt  and
      Callison-Burch, Chris  and
      Osborne, Miles",
    booktitle = "Proceedings of the Seventh Workshop on Statistical Machine Translation",
    month = jun,
    year = "2012",
    address = "Montr{\'e}al, Canada",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/W12-3152",
    pages = "401--409",
}"""
    url = 'https://github.com/joshua-decoder/indian-parallel-corpora/archive/a2cd1a99.tar.gz'
    l2 = 'en'
    langs = ['ml', 'hi', 'ur', 'bn', 'te', 'ta']
    for l1 in langs:
        for split in ['training', 'dev', 'test', 'devtest', 'dict']:
            if l1 == 'hi' and split == 'dict':
                continue   # hindi dont have dict
            f1 = f'*/{l1}-{l2}/{split}.{l1}-{l2}.{l1}'
            f2 = f'*/{l1}-{l2}/{split}.{l1}-{l2}.{l2}'
            if split not in ('training', 'dict'):
                f2 += '.0'
            ent = Entry(langs=(l1, l2), url=url, name=f'JoshuaIndianCorpus_{split}',
                  filename='joshua-indian-parallel-corpora.tar.gz',
                  in_paths=[f1, f2], in_ext='txt', cite=cite)
            index.add_entry(ent)



#!/usr/bin/env python
#
# Author: Thamme Gowda [tg (at) isi (dot) edu] 
# Created: 5/13/20
from pathlib import Path
from mtdata.utils import IO, log
from xml.etree import ElementTree as ET
import collections as coll
import re


class OpusXcesParser:

    @classmethod
    def read_alignments(cls, align_file: Path):
        assert align_file.is_file(), f'{align_file} not found'
        with IO.reader(align_file) as reader:
            context = ET.iterparse(reader, events=['end'])
            docs = (el for event, el in context if el.tag == 'linkGrp')
            count = 0
            skip_count = 0
            for doc in docs:
                algn = []
                doc_parse = {'src_doc': doc.attrib['fromDoc'], 'tgt_doc': doc.attrib['toDoc'],
                             'align': algn}
                for seg in doc.findall('.//link'):
                    parts = seg.attrib.get('xtargets', ';').strip().split(';')
                    if len(parts) != 2:
                        skip_count += 1
                        continue
                    src_ids, tgt_ids = parts
                    src_ids, tgt_ids = src_ids.split(), tgt_ids.split()
                    confidence = float(seg.attrib.get('certainty', '-inf'))
                    algn.append((confidence, src_ids, tgt_ids))
                yield doc_parse
                doc.clear()
                count += 1
            log.info(f"read {count} docs from {align_file}")

    @classmethod
    def read_doc(cls, data):
        mem = {}
        context = ET.iterparse(data, events=['end'])
        segs = (el for event, el in context if el.tag == 's')
        for seg in segs:
            seg_id = seg.attrib['id']
            mem[seg_id] = ' '.join(w.text for w in seg.findall('.//w'))
        return mem

    @classmethod
    def read(cls, align_file: Path, l1_dir: Path, l2_dir: Path, name='JW300', min_confidence=0.01):
        doc_aligns = cls.read_alignments(align_file)
        assert l1_dir.is_file() and l1_dir.suffix == '.zip', f'{l1_dir}'
        assert l2_dir.is_file() and l2_dir.suffix == '.zip', f'{l2_dir}'
        from zipfile import ZipFile
        stats = coll.defaultdict(int)

        with ZipFile(l1_dir) as l1_zip, ZipFile(l2_dir) as l2_zip:
            l1_doc_names = set(l1_zip.namelist())
            l2_doc_names = set(l2_zip.namelist())
            for d in doc_aligns:
                src_doc_path = f'{name}/xml/' + re.sub(r'\.gz$', '', d['src_doc'])
                tgt_doc_path = f'{name}/xml/' + re.sub(r'\.gz$', '', d['tgt_doc'])
                if src_doc_path not in l1_doc_names or tgt_doc_path not in l2_doc_names:
                    stats['doc_not_found'] += 1
                    continue
                with l1_zip.open(src_doc_path) as f1:
                    src_doc = cls.read_doc(f1)
                with l2_zip.open(tgt_doc_path) as f2:
                    tgt_doc = cls.read_doc(f2)

                for confidence, src_ids, tgt_ids in d['align']:
                    if not src_ids or not tgt_ids:
                        stats['missing_seg_skips'] += 1
                        continue
                    if confidence < min_confidence:
                        stats['low_conf_skips'] += 1
                        continue

                    src_segs = [src_doc.get(id, None) for id in src_ids]
                    tgt_segs = [tgt_doc.get(id, None) for id in tgt_ids]
                    if not all(src_segs) or not all(tgt_segs):
                        stats['error_seg_skips'] += 1
                        continue
                    src_seg = ' '.join(src_segs)
                    tgt_seg = ' '.join(tgt_segs)
                    yield src_seg, tgt_seg
                    stats['success'] += 1
        stats = dict(stats)
        log.info(f"Read aligned segments from {l1_dir}, {l2_dir} {stats}")
